import re

class DisambiguatorPrefixRule40a(object):
    """Disambiguate Prefix Rule 40a (CC infix rules)
    Rule 40a : CinV -> CinV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 40a (CC infix rules)
        Rule 40a : CinV -> CinV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])(in[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)

class DisambiguatorPrefixRule40b(object):
    """Disambiguate Prefix Rule 40b (CC infix rules)
    Rule 40b : CinV -> CV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 40b (CC infix rules)
        Rule 40b : CinV -> CV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])in([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule5(object):
    """Disambiguate Prefix Rule 5
    Rule 5 : beC1erC2 -> be-C1erC2 where C1 != 'r'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 5
        Rule 5 : beC1erC2 -> be-C1erC2 where C1 != 'r'
        """
        matches = re.match(r'be([bcdfghjklmnpqstvwxyz])(er[bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule12(object):
    """Disambiguate Prefix Rule 12
    Nazief and Adriani Rule 12 : beC1erC2 -> be-C1erC2 where C1 != 'r'
    Modified by Jelita Asian's CS Rule 12 : mempe -> mem-pe to stem mempengaruhi
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 12
        Nazief and Adriani Rule 12 : beC1erC2 -> be-C1erC2 where C1 != 'r'
        Modified by Jelita Asian's CS Rule 12 : mempe -> mem-pe to stem mempengaruhi
        """
        matches = re.match(r'^mempe(.*)$', word)
        if matches:
            return 'pe' + matches.group(1)



import re

class DisambiguatorPrefixRule41(object):
    """Disambiguate Prefix Rule 41
    Rule 41 : kuA -> ku-A
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 41
        Rule 41 : kuA -> ku-A
        """
        matches = re.match(r'^ku(.*)$', word)
        if matches:
            return matches.group(1)



import re

class DisambiguatorPrefixRule17a(object):
    """Disambiguate Prefix Rule 17a
    Rule 17a : mengV -> meng-V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 17a
        Rule 17a : mengV -> meng-V
        """
        matches = re.match(r'^meng([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule17b(object):
    """Disambiguate Prefix Rule 17b
    Rule 17b : mengV -> meng-kV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 17b
        Rule 17b : mengV -> meng-kV
        """
        matches = re.match(r'^meng([aiueo])(.*)$', word)
        if matches:
            return 'k' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule17c(object):
    """Disambiguate Prefix Rule 17c
    Rule 17c : mengV -> meng-V- where V = 'e'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 17c
        Rule 17c : mengV -> meng-V- where V = 'e'
        """
        matches = re.match(r'^menge(.*)$', word)
        if matches:
            return matches.group(1)

class DisambiguatorPrefixRule17d(object):
    """Disambiguate Prefix Rule 17d
    Rule 17d : mengV -> me-ngV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 17d
        Rule 17d : mengV -> me-ngV
        """
        matches = re.match(r'^meng([aiueo])(.*)$', word)
        if matches:
            return 'ng' + matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule7(object):
    """Disambiguate Prefix Rule 7
    Rule 7 : terCerv -> ter-CerV where C != 'r'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 7
        Rule 7 : terCerv -> ter-CerV where C != 'r'
        """
        matches = re.match(r'^ter([bcdfghjklmnpqrstvwxyz])er([aiueo].*)$', word)
        if matches:
            if matches.group(1) == 'r':
                return
            return matches.group(1) + 'er' + matches.group(2)



import re

class DisambiguatorPrefixRule29(object):
    """Disambiguate Prefix Rule 29
    Original Rule 29 : peng{g|h|q} -> peng-{g|h|q}
    Modified Rule 29 by ECS : pengC -> peng-C
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 29
        Original Rule 29 : peng{g|h|q} -> peng-{g|h|q}
        Modified Rule 29 by ECS : pengC -> peng-C
        """
        matches = re.match(r'^peng([bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule14(object):
    """Disambiguate Prefix Rule 14
    Rule 14 modified by Andy Librian : men{c|d|j|s|t|z} -> men-{c|d|j|s|t|z}
    in order to stem mentaati
  
    Rule 14 modified by ECS: men{c|d|j|s|z} -> men-{c|d|j|s|z}
    in order to stem mensyaratkan, mensyukuri
  
    Original CS Rule no 14 was : men{c|d|j|z} -> men-{c|d|j|z}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 14
        Rule 14 modified by Andy Librian : men{c|d|j|s|t|z} -> men-{c|d|j|s|t|z}
        in order to stem mentaati
  
        Rule 14 modified by ECS: men{c|d|j|s|z} -> men-{c|d|j|s|z}
        in order to stem mensyaratkan, mensyukuri
  
        Original CS Rule no 14 was : men{c|d|j|z} -> men-{c|d|j|z}
        """
        matches = re.match(r'^men([cdjstz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



class DisambiguatorPrefixRule4(object):
    """Disambiguate Prefix Rule 4
    Rule 4 : belajar -> bel-ajar
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 4
        Rule 4 : belajar -> bel-ajar
        """
        if word == 'belajar':
            return 'ajar'



import re

class DisambiguatorPrefixRule42(object):
    """Disambiguate Prefix Rule 42
    Rule 42 : kauA -> kau-A
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 42
        Rule 42 : kauA -> kau-A
        """
        matches = re.match(r'^kau(.*)$', word)
        if matches:
            return matches.group(1)



import re

class PrecedenceAdjustmentSpecification(object):
    """Confix Stripping Rule Precedence Adjustment Specification.
    Asian J. (2007) "Effective Techniques for Indonesian Text Retrieval" page 78-79.

    @link   http://researchbank.rmit.edu.au/eserv/rmit:6312/Asian.pdf
    """

    def is_satisfied_by(self, value):
        regex_rules = [
            r'^be(.*)lah$',
            r'^be(.*)an$',
            r'^me(.*)i$',
            r'^di(.*)i$',
            r'^pe(.*)i$',
            r'^ter(.*)i$',
        ]

        for rule in regex_rules:
            if re.match(rule, value):
                return True

        return False






import re

class DisambiguatorPrefixRule30a(object):
    """Disambiguate Prefix Rule 30a
    Rule 30a : pengV -> peng-V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 30a
        Rule 30a : pengV -> peng-V
        """
        matches = re.match(r'^peng([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule30b(object):
    """Disambiguate Prefix Rule 30b
    Rule 30a : pengV -> peng-kV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 30b
        Rule 30a : pengV -> peng-kV
        """
        matches = re.match(r'^peng([aiueo])(.*)$', word)
        if matches:
            return 'k' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule30c(object):
    """Disambiguate Prefix Rule 30c
    Rule 30a : pengV -> pengV- where V = 'e'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 30c
        Rule 30a : pengV -> pengV- where V = 'e'
        """
        matches = re.match(r'^penge(.*)$', word)
        if matches:
            return matches.group(1)



import re

class DisambiguatorPrefixRule34(object):
    """Disambiguate Prefix Rule 34
    Rule 34 : peCP -> pe-CP where C != {r|w|y|l|m|n} and P != 'er'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 34
        Rule 34 : peCP -> pe-CP where C != {r|w|y|l|m|n} and P != 'er'
        """
        matches = re.match(r'^pe([bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            if re.match(r'^er(.*)$', matches.group(2)):
                return
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule2(object):
    """Disambiguate Prefix Rule 2
    Rule 2 : berCAP -> ber-CAP where C != 'r' AND P != 'er'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 2
        Rule 2 : berCAP -> ber-CAP where C != 'r' AND P != 'er'
        """
        matches = re.match(r'^ber([bcdfghjklmnpqrstvwxyz])([a-z])(.*)', word)
        if matches:
            if re.match(r'^er(.*)$', matches.group(3)):
                return
            
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule1a(object):
    """Disambiguate Prefix Rule 1a
    Rule 1a : berV -> ber-V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 1a
        Rule 1a : berV -> ber-V
        """
        matches = re.match(r'^ber([aiueo].*)$', word)
        if matches:
            return matches.group(1)

class DisambiguatorPrefixRule1b(object):
    """Disambiguate Prefix Rule 1b
    Rule 1b : berV -> be-rV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 1b
        Rule 1b : berV -> be-rV
        """
        matches = re.match(r'^ber([aiueo].*)$', word)
        if matches:
            return 'r' + matches.group(1)



import re

def normalize_text(text):
    result = text.lower() #lower the text even unicode given
    result = re.sub(r'[^a-z0-9 -]', ' ', result, flags = re.IGNORECASE|re.MULTILINE)
    result = re.sub(r'( +)', ' ', result, flags = re.IGNORECASE|re.MULTILINE)

    return result.strip()






import re
from Sastrawi.Stemmer.Context.Visitor.VisitorProvider import VisitorProvider
from Sastrawi.Stemmer.Filter import TextNormalizer
from Sastrawi.Stemmer.Context.Context import Context

class Stemmer(object):
    """Indonesian Stemmer.
    Nazief & Adriani, CS Stemmer, ECS Stemmer, Improved ECS.

    @link https://github.com/sastrawi/sastrawi/wiki/Resources
    """
    def __init__(self, dictionary):
        self.dictionary = dictionary
        self.visitor_provider = VisitorProvider()

    def get_dictionary(self):
        return self.dictionary

    def stem(self, text):
        """Stem a text string to its common stem form."""
        normalizedText = TextNormalizer.normalize_text(text)

        words = normalizedText.split(' ')
        stems = []

        for word in words:
            stems.append(self.stem_word(word))

        return ' '.join(stems)

    def stem_word(self, word):
        """Stem a word to its common stem form."""
        if self.is_plural(word):
            return self.stem_plural_word(word)
        else:
            return self.stem_singular_word(word)

    def is_plural(self, word):
        #-ku|-mu|-nya
        #nikmat-Ku, etc
        matches = re.match(r'^(.*)-(ku|mu|nya|lah|kah|tah|pun)$', word)
        if matches:
            return matches.group(1).find('-') != -1

        return word.find('-') != -1

    def stem_plural_word(self, plural):
        """Stem a plural word to its common stem form.
        Asian J. (2007) "Effective Techniques for Indonesian Text Retrieval" page 76-77.

        @link   http://researchbank.rmit.edu.au/eserv/rmit:6312/Asian.pdf
        """
        matches = re.match(r'^(.*)-(.*)$', plural)
        #translated from PHP conditional check:
        #if (!isset($words[1]) || !isset($words[2]))
        if not matches:
            return plural
        words = [matches.group(1), matches.group(2)]

        #malaikat-malaikat-nya -> malaikat malaikat-nya
        suffix = words[1]
        suffixes = ['ku', 'mu', 'nya', 'lah', 'kah', 'tah', 'pun']
        matches = re.match(r'^(.*)-(.*)$', words[0])
        if suffix in suffixes and matches:
            words[0] = matches.group(1)
            words[1] = matches.group(2) + '-' + suffix

        #berbalas-balasan -> balas
        rootWord1 = self.stem_singular_word(words[0])
        rootWord2 = self.stem_singular_word(words[1])

        #meniru-nirukan -> tiru
        if not self.dictionary.contains(words[1]) and rootWord2 == words[1]:
            rootWord2 = self.stem_singular_word('me' + words[1])

        if rootWord1 == rootWord2:
            return rootWord1
        else:
            return plural

    def stem_singular_word(self, word):
        """Stem a singular word to its common stem form."""
        context = Context(word, self.dictionary, self.visitor_provider)
        context.execute()

        return context.result




import re


class DisambiguatorPrefixRule24(object):
    """Disambiguate Prefix Rule 24
    Rule 24 : perCAerV -> per-CAerV where C != 'r'
    """

    def disambiguate(self, word):
        """Disambiguate Prefix Rule 24
        Rule 24 : perCAerV -> per-CAerV where C != 'r'
        """
        matches = re.match(r'^per([bcdfghjklmnpqrstvwxyz])([a-z])er([aiueo])(.*)$', word)
        if matches:
            if matches.group(1) == 'r':
                return
            return matches.group(1) + matches.group(2) + 'er' + matches.group(3) + matches.group(4)



import re

class DisambiguatorPrefixRule37a(object):
    """Disambiguate Prefix Rule 37a (CC infix rules)
    Rule 37a : CerV -> CerV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 37a (CC infix rules)
        Rule 37a : CerV -> CerV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])(er[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)

class DisambiguatorPrefixRule37b(object):
    """Disambiguate Prefix Rule 37b (CC infix rules)
    Rule 37b : CerV -> CV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 37b (CC infix rules)
        Rule 37b : CerV -> CV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])er([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re
from Sastrawi.Stemmer.Context.Removal import Removal

class RemoveDerivationalSuffix(object):
    """Remove Derivational Suffix.
    Asian J. (2007) "Effective Techniques for Indonesian Text Retrieval". page 61

    @link http://researchbank.rmit.edu.au/eserv/rmit:6312/Asian.pdf
    """

    def visit(self, context):
        result = self.remove(context.current_word)
        if result != context.current_word:
            removedPart = re.sub(result, '', context.current_word, 1)

            removal = Removal(self, context.current_word, result, removedPart, 'DS')

            context.add_removal(removal)
            context.current_word = result

    def remove(self, word):
        """Remove derivational suffix
        Original rule : i|kan|an
        Added the adopted foreign suffix rule : is|isme|isasi
        """
        return re.sub(r'(is|isme|isasi|i|kan|an)$', '', word, 1)





import re

class DisambiguatorPrefixRule9(object):
    """Disambiguate Prefix Rule 9
    Rule 9 : te-C1erC2 -> te-C1erC2 where C1 != 'r'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 9
        Rule 9 : te-C1erC2 -> te-C1erC2 where C1 != 'r'
        """
        matches = re.match(r'^te([bcdfghjklmnpqrstvwxyz])er([bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            if matches.group(1) == 'r':
                return
            return matches.group(1) + 'er' + matches.group(2) + matches.group(3)



import re
from Sastrawi.Stemmer.Context.Removal import Removal

class AbstractDisambiguatePrefixRule(object):
    """description of class"""

    def __init__(self):
        self.disambiguators = []

    def visit(self, context):
        result = None

        for disambiguator in self.disambiguators:
            result = disambiguator.disambiguate(context.current_word)
            if context.dictionary.contains(result):
                break

        if not result:
            return

        removedPart = re.sub(result, '', context.current_word, 1)

        removal = Removal(self, context.current_word, result, removedPart, 'DP')

        context.add_removal(removal)
        context.current_word = result

    def add_disambiguators(self, disambiguators):
        for disambiguator in disambiguators:
            self.add_disambiguator(disambiguator)

    def add_disambiguator(self, disambiguator):
        self.disambiguators.append(disambiguator)






class ContextInterface(object):
    """description of class"""

    def getOriginalWord(self):
        pass

    def setCurrentWord(self, word):
        pass

    def getCurrentWord(self):
        pass

    def getDictionary(self):
        pass

    def stopProcess(self):
        pass

    def processIsStopped(self):
        pass

    def addRemoval(self, removal):
        pass

    def getRemovals(self):
        pass







from Sastrawi.Stemmer.Context.RemovalInterface import RemovalInterface

class Removal(RemovalInterface):
    """description of class"""

    def __init__(self, visitor, subject, result, removedPart, affixType):
        self.visitor = visitor
        self.subject = subject
        self.result = result
        self.removedPart = removedPart
        self.affixType = affixType

    def get_visitor(self):
        return self.visitor

    def get_subject(self):
        return self.subject

    def get_result(self):
        return self.result

    def get_removed_part(self):
        return self.removedPart

    def get_affix_type(self):
        return self.affixType






import re

class DisambiguatorPrefixRule31a(object):
    """Disambiguate Prefix Rule 31a
    Rule 31a : penyV -> pe-nyV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 31a
        Rule 31a : penyV -> pe-nyV
        """
        matches = re.match(r'^peny([aiueo])(.*)$', word)
        if matches:
            return 'ny' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule31b(object):
    """Disambiguate Prefix Rule 31b
    Original Rule 31 : penyV -> peny-sV
    Modified by CC, shifted to 31b
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 31b
        Original Rule 31 : penyV -> peny-sV
        Modified by CC, shifted to 31b
        """
        matches = re.match(r'^peny([aiueo])(.*)$', word)
        if matches:
            return 's' + matches.group(1) + matches.group(2)



class RemovalInterface(object):
    """description of class"""

    def get_visitor(self):
        pass

    def get_subject(self):
        pass

    def get_result(self):
        pass

    def get_removed_part(self):
        pass

    def get_affix_type(self):
        pass







from Sastrawi.Stemmer.Cache.CacheInterface import CacheInterface

class ArrayCache(CacheInterface):
    """description of class"""

    def __init__(self):
        self.data = {}

    def set(self, key, value):
        self.data[key] = value

    def get(self, key):
        if key in self.data:
            return self.data[key]

    def has(self, key):
        return key in self.data





from Sastrawi.Stemmer.Context.Visitor.AbstractDisambiguatePrefixRule import AbstractDisambiguatePrefixRule

class PrefixDisambiguator(AbstractDisambiguatePrefixRule):
    """description of class"""

    def __init__(self, disambiguators):
        super(PrefixDisambiguator, self).__init__()

        self.add_disambiguators(disambiguators)






from Sastrawi.Stemmer.Context.Visitor.DontStemShortWord import DontStemShortWord
from Sastrawi.Stemmer.Context.Visitor.RemoveInflectionalParticle import RemoveInflectionalParticle
from Sastrawi.Stemmer.Context.Visitor.RemoveDerivationalSuffix import RemoveDerivationalSuffix
from Sastrawi.Stemmer.Context.Visitor.RemoveInflectionalPossessivePronoun import RemoveInflectionalPossessivePronoun
from Sastrawi.Stemmer.Context.Visitor.PrefixDisambiguator import PrefixDisambiguator
from Sastrawi.Stemmer.Context.Visitor.RemovePlainPrefix import RemovePlainPrefix

from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule1 import DisambiguatorPrefixRule1a, DisambiguatorPrefixRule1b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule2 import DisambiguatorPrefixRule2
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule3 import DisambiguatorPrefixRule3
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule4 import DisambiguatorPrefixRule4
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule5 import DisambiguatorPrefixRule5
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule6 import DisambiguatorPrefixRule6a, DisambiguatorPrefixRule6b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule7 import DisambiguatorPrefixRule7
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule8 import DisambiguatorPrefixRule8
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule9 import DisambiguatorPrefixRule9
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule10 import DisambiguatorPrefixRule10
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule11 import DisambiguatorPrefixRule11
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule12 import DisambiguatorPrefixRule12
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule13 import DisambiguatorPrefixRule13a, DisambiguatorPrefixRule13b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule14 import DisambiguatorPrefixRule14
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule15 import DisambiguatorPrefixRule15a, DisambiguatorPrefixRule15b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule16 import DisambiguatorPrefixRule16
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule17 import DisambiguatorPrefixRule17a, DisambiguatorPrefixRule17b, DisambiguatorPrefixRule17c, DisambiguatorPrefixRule17d
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule18 import DisambiguatorPrefixRule18a, DisambiguatorPrefixRule18b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule19 import DisambiguatorPrefixRule19
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule20 import DisambiguatorPrefixRule20
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule21 import DisambiguatorPrefixRule21a, DisambiguatorPrefixRule21b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule23 import DisambiguatorPrefixRule23
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule24 import DisambiguatorPrefixRule24
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule25 import DisambiguatorPrefixRule25
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule26 import DisambiguatorPrefixRule26a, DisambiguatorPrefixRule26b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule27 import DisambiguatorPrefixRule27
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule28 import DisambiguatorPrefixRule28a, DisambiguatorPrefixRule28b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule29 import DisambiguatorPrefixRule29
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule30 import DisambiguatorPrefixRule30a, DisambiguatorPrefixRule30b, DisambiguatorPrefixRule30c
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule31 import DisambiguatorPrefixRule31a, DisambiguatorPrefixRule31b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule32 import DisambiguatorPrefixRule32
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule34 import DisambiguatorPrefixRule34
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule35 import DisambiguatorPrefixRule35
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule36 import DisambiguatorPrefixRule36
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule37 import DisambiguatorPrefixRule37a, DisambiguatorPrefixRule37b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule38 import DisambiguatorPrefixRule38a, DisambiguatorPrefixRule38b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule39 import DisambiguatorPrefixRule39a, DisambiguatorPrefixRule39b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule40 import DisambiguatorPrefixRule40a, DisambiguatorPrefixRule40b
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule41 import DisambiguatorPrefixRule41
from Sastrawi.Morphology.Disambiguator.DisambiguatorPrefixRule42 import DisambiguatorPrefixRule42

class VisitorProvider(object):
    """description of class"""

    def __init__(self):
        self.visitors = []
        self.suffix_visitors = []
        self.prefix_pisitors = []

        self.init_visitors()

    def init_visitors(self):
        self.visitors.append(DontStemShortWord())

        #{lah|kah|tah|pun}
        self.suffix_visitors.append(RemoveInflectionalParticle())
        #{ku|mu|nya}
        self.suffix_visitors.append(RemoveInflectionalPossessivePronoun())
        #{i|kan|an}
        self.suffix_visitors.append(RemoveDerivationalSuffix())

        #{di|ke|se}
        self.prefix_pisitors.append(RemovePlainPrefix())

        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule1a(), DisambiguatorPrefixRule1b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule2()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule3()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule4()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule5()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule6a(), DisambiguatorPrefixRule6b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule7()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule8()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule9()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule10()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule11()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule12()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule13a(), DisambiguatorPrefixRule13b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule14()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule15a(), DisambiguatorPrefixRule15b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule16()]))

        disambiguators17 = [DisambiguatorPrefixRule17a(), DisambiguatorPrefixRule17b(), DisambiguatorPrefixRule17c(),
                            DisambiguatorPrefixRule17d()]
        self.prefix_pisitors.append(PrefixDisambiguator(disambiguators17))

        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule18a(), DisambiguatorPrefixRule18b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule19()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule20()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule21a(), DisambiguatorPrefixRule21b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule23()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule24()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule25()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule26a(), DisambiguatorPrefixRule26b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule27()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule28a(), DisambiguatorPrefixRule28b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule29()]))

        disambiguators30 = [DisambiguatorPrefixRule30a(), DisambiguatorPrefixRule30b(), DisambiguatorPrefixRule30c()]
        self.prefix_pisitors.append(PrefixDisambiguator(disambiguators30))

        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule31a(), DisambiguatorPrefixRule31b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule32()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule34()]))

        #CS additional rules
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule35()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule36()]))

        #CS infix rules
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule37a(), DisambiguatorPrefixRule37b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule38a(), DisambiguatorPrefixRule38b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule39a(), DisambiguatorPrefixRule39b()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule40a(), DisambiguatorPrefixRule40b()]))

        #Sastrawi rules
        #ku-A, kau-A
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule41()]))
        self.prefix_pisitors.append(PrefixDisambiguator([DisambiguatorPrefixRule42()]))

    def get_visitors(self):
        return self.visitors

    def get_suffix_visitors(self):
        return self.suffix_visitors

    def get_prefix_visitors(self):
        return self.prefix_pisitors




from Sastrawi.Dictionary.ArrayDictionary import ArrayDictionary
from Sastrawi.StopWordRemover.StopWordRemover import StopWordRemover

class StopWordRemoverFactory(object):
    """description of class"""

    def create_stop_word_remover(self):
        stopWords = self.get_stop_words()
        dictionary = ArrayDictionary(stopWords)
        stopWordRemover = StopWordRemover(dictionary)

        return stopWordRemover

    def get_stop_words(self):
        return ['a','ada','adalah','adanya','adapun','agak','agaknya','agar','akan','akankah','akhir',
            'akhiri','akhirnya','aku','akulah','amat','amatlah','anda','andalah','antar','antara',
            'antaranya','apa','apaan','apabila','apakah','apalagi','apatah','arti','artinya','asal',
            'asalkan','atas','atau','ataukah','ataupun','awal','awalnya','b','bagai','bagaikan',
            'bagaimana','bagaimanakah','bagaimanapun','bagainamakah','bagi','bagian','bahkan','bahwa',
            'bahwasannya','bahwasanya','baik','baiklah','bakal','bakalan','balik','banyak','bapak',
            'baru','bawah','beberapa','begini','beginian','beginikah','beginilah','begitu','begitukah',
            'begitulah','begitupun','bekerja','belakang','belakangan','belum','belumlah','benar',
            'benarkah','benarlah','berada','berakhir','berakhirlah','berakhirnya','berapa','berapakah',
            'berapalah','berapapun','berarti','berawal','berbagai','berdatangan','beri','berikan',
            'berikut','berikutnya','berjumlah','berkali-kali','berkata','berkehendak','berkeinginan',
            'berkenaan','berlainan','berlalu','berlangsung','berlebihan','bermacam','bermacam-macam',
            'bermaksud','bermula','bersama','bersama-sama','bersiap','bersiap-siap','bertanya',
            'bertanya-tanya','berturut','berturut-turut','bertutur','berujar','berupa','besar',
            'betul','betulkah','biasa','biasanya','bila','bilakah','bisa','bisakah','boleh','bolehkah',
            'bolehlah','buat','bukan','bukankah','bukanlah','bukannya','bulan','bung','c','cara',
            'caranya','cukup','cukupkah','cukuplah','cuma','d','dahulu','dalam','dan','dapat','dari',
            'daripada','datang','dekat','demi','demikian','demikianlah','dengan','depan','di','dia',
            'diakhiri','diakhirinya','dialah','diantara','diantaranya','diberi','diberikan','diberikannya',
            'dibuat','dibuatnya','didapat','didatangkan','digunakan','diibaratkan','diibaratkannya',
            'diingat','diingatkan','diinginkan','dijawab','dijelaskan','dijelaskannya','dikarenakan',
            'dikatakan','dikatakannya','dikerjakan','diketahui','diketahuinya','dikira','dilakukan',
            'dilalui','dilihat','dimaksud','dimaksudkan','dimaksudkannya','dimaksudnya','diminta',
            'dimintai','dimisalkan','dimulai','dimulailah','dimulainya','dimungkinkan','dini','dipastikan',
            'diperbuat','diperbuatnya','dipergunakan','diperkirakan','diperlihatkan','diperlukan',
            'diperlukannya','dipersoalkan','dipertanyakan','dipunyai','diri','dirinya','disampaikan',
            'disebut','disebutkan','disebutkannya','disini','disinilah','ditambahkan','ditandaskan',
            'ditanya','ditanyai','ditanyakan','ditegaskan','ditujukan','ditunjuk','ditunjuki','ditunjukkan',
            'ditunjukkannya','ditunjuknya','dituturkan','dituturkannya','diucapkan','diucapkannya',
            'diungkapkan','dong','dua','dulu','e','empat','enak','enggak','enggaknya','entah','entahlah',
            'f','g','guna','gunakan','h','hadap','hai','hal','halo','hallo','hampir','hanya','hanyalah',
            'hari','harus','haruslah','harusnya','helo','hello','hendak','hendaklah','hendaknya','hingga',
            'i','ia','ialah','ibarat','ibaratkan','ibaratnya','ibu','ikut','ingat','ingat-ingat','ingin',
            'inginkah','inginkan','ini','inikah','inilah','itu','itukah','itulah','j','jadi','jadilah',
            'jadinya','jangan','jangankan','janganlah','jauh','jawab','jawaban','jawabnya','jelas',
            'jelaskan','jelaslah','jelasnya','jika','jikalau','juga','jumlah','jumlahnya','justru',
            'k','kadar','kala','kalau','kalaulah','kalaupun','kali','kalian','kami','kamilah','kamu',
            'kamulah','kan','kapan','kapankah','kapanpun','karena','karenanya','kasus','kata','katakan',
            'katakanlah','katanya','ke','keadaan','kebetulan','kecil','kedua','keduanya','keinginan',
            'kelamaan','kelihatan','kelihatannya','kelima','keluar','kembali','kemudian','kemungkinan',
            'kemungkinannya','kena','kenapa','kepada','kepadanya','kerja','kesampaian','keseluruhan',
            'keseluruhannya','keterlaluan','ketika','khusus','khususnya','kini','kinilah','kira',
            'kira-kira','kiranya','kita','kitalah','kok','kurang','l','lagi','lagian','lah','lain',
            'lainnya','laku','lalu','lama','lamanya','langsung','lanjut','lanjutnya','lebih','lewat',
            'lihat','lima','luar','m','macam','maka','makanya','makin','maksud','malah','malahan',
            'mampu','mampukah','mana','manakala','manalagi','masa','masalah','masalahnya','masih',
            'masihkah','masing','masing-masing','masuk','mata','mau','maupun','melainkan','melakukan',
            'melalui','melihat','melihatnya','memang','memastikan','memberi','memberikan','membuat',
            'memerlukan','memihak','meminta','memintakan','memisalkan','memperbuat','mempergunakan',
            'memperkirakan','memperlihatkan','mempersiapkan','mempersoalkan','mempertanyakan','mempunyai',
            'memulai','memungkinkan','menaiki','menambahkan','menandaskan','menanti','menanti-nanti',
            'menantikan','menanya','menanyai','menanyakan','mendapat','mendapatkan','mendatang','mendatangi',
            'mendatangkan','menegaskan','mengakhiri','mengapa','mengatakan','mengatakannya','mengenai',
            'mengerjakan','mengetahui','menggunakan','menghendaki','mengibaratkan','mengibaratkannya',
            'mengingat','mengingatkan','menginginkan','mengira','mengucapkan','mengucapkannya','mengungkapkan',
            'menjadi','menjawab','menjelaskan','menuju','menunjuk','menunjuki','menunjukkan','menunjuknya',
            'menurut','menuturkan','menyampaikan','menyangkut','menyatakan','menyebutkan','menyeluruh',
            'menyiapkan','merasa','mereka','merekalah','merupakan','meski','meskipun','meyakini','meyakinkan',
            'minta','mirip','misal','misalkan','misalnya','mohon','mula','mulai','mulailah','mulanya','mungkin',
            'mungkinkah','n','nah','naik','namun','nanti','nantinya','nya','nyaris','nyata','nyatanya',
            'o','oleh','olehnya','orang','p','pada','padahal','padanya','pak','paling','panjang','pantas',
            'para','pasti','pastilah','penting','pentingnya','per','percuma','perlu','perlukah','perlunya',
            'pernah','persoalan','pertama','pertama-tama','pertanyaan','pertanyakan','pihak','pihaknya',
            'pukul','pula','pun','punya','q','r','rasa','rasanya','rupa','rupanya','s','saat','saatnya','saja',
            'sajalah','salam','saling','sama','sama-sama','sambil','sampai','sampai-sampai','sampaikan','sana',
            'sangat','sangatlah','sangkut','satu','saya','sayalah','se','sebab','sebabnya','sebagai',
            'sebagaimana','sebagainya','sebagian','sebaik','sebaik-baiknya','sebaiknya','sebaliknya',
            'sebanyak','sebegini','sebegitu','sebelum','sebelumnya','sebenarnya','seberapa','sebesar',
            'sebetulnya','sebisanya','sebuah','sebut','sebutlah','sebutnya','secara','secukupnya','sedang',
            'sedangkan','sedemikian','sedikit','sedikitnya','seenaknya','segala','segalanya','segera',
            'seharusnya','sehingga','seingat','sejak','sejauh','sejenak','sejumlah','sekadar','sekadarnya',
            'sekali','sekali-kali','sekalian','sekaligus','sekalipun','sekarang','sekaranglah','sekecil',
            'seketika','sekiranya','sekitar','sekitarnya','sekurang-kurangnya','sekurangnya','sela','selain',
            'selaku','selalu','selama','selama-lamanya','selamanya','selanjutnya','seluruh','seluruhnya',
            'semacam','semakin','semampu','semampunya','semasa','semasih','semata','semata-mata','semaunya',
            'sementara','semisal','semisalnya','sempat','semua','semuanya','semula','sendiri','sendirian',
            'sendirinya','seolah','seolah-olah','seorang','sepanjang','sepantasnya','sepantasnyalah',
            'seperlunya','seperti','sepertinya','sepihak','sering','seringnya','serta','serupa','sesaat',
            'sesama','sesampai','sesegera','sesekali','seseorang','sesuatu','sesuatunya','sesudah',
            'sesudahnya','setelah','setempat','setengah','seterusnya','setiap','setiba','setibanya',
            'setidak-tidaknya','setidaknya','setinggi','seusai','sewaktu','siap','siapa','siapakah',
            'siapapun','sini','sinilah','soal','soalnya','suatu','sudah','sudahkah','sudahlah','supaya',
            't','tadi','tadinya','tahu','tak','tambah','tambahnya','tampak','tampaknya','tandas','tandasnya',
            'tanpa','tanya','tanyakan','tanyanya','tapi','tegas','tegasnya','telah','tempat','tentang','tentu',
            'tentulah','tentunya','tepat','terakhir','terasa','terbanyak','terdahulu','terdapat','terdiri',
            'terhadap','terhadapnya','teringat','teringat-ingat','terjadi','terjadilah','terjadinya','terkira',
            'terlalu','terlebih','terlihat','termasuk','ternyata','tersampaikan','tersebut','tersebutlah',
            'tertentu','tertuju','terus','terutama','tetap','tetapi','tiap','tiba','tiba-tiba','tidak',
            'tidakkah','tidaklah','tiga','toh','tuju','tunjuk','turut','tutur','tuturnya','u','ucap','ucapnya',
            'ujar','ujarnya','umumnya','ungkap','ungkapnya','untuk','usah','usai','v','w','waduh','wah','wahai',
            'waktunya','walau','walaupun','wong','x','y','ya','yaitu','yakin','yakni','yang','z']







import re

class DisambiguatorPrefixRule13a(object):
    """Disambiguate Prefix Rule 13a
    Rule 13a : mem{rV|V} -> me-m{rV|V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 13a
        Rule 13a : mem{rV|V} -> me-m{rV|V}
        """
        matches = re.match(r'^mem([aiueo])(.*)$', word)
        if matches:
            return 'm' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule13b(object):
    """Disambiguate Prefix Rule 13b
    Rule 13b : mem{rV|V} -> me-p{rV|V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 13b
        Rule 13b : mem{rV|V} -> me-p{rV|V}
        """
        matches = re.match(r'^mem([aiueo])(.*)$', word)
        if matches:
            return 'p' + matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule32(object):
    """Disambiguate Prefix Rule 32
    Rule 32 : pelV -> pe-lV except pelajar -> ajar
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 32
        Rule 32 : pelV -> pe-lV except pelajar -> ajar
        """
        if word == 'pelajar':
            return 'ajar'
        matches = re.match(r'^pe(l[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



class DontStemShortWord(object):
    """description of class"""

    def visit(self, context):
        if self.is_whort_word(context.current_word):
            context.stopProcess()

    def is_whort_word(self, word):
        return len(word) <= 3







import re

class DisambiguatorPrefixRule15a(object):
    """Disambiguate Prefix Rule 15a
    Rule 15a : men{V} -> me-n{V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 15a
        Rule 15a : men{V} -> me-n{V}
        """
        matches = re.match(r'^men([aiueo])(.*)$', word)
        if matches:
            return 'n' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule15b(object):
    """Disambiguate Prefix Rule 15b
    Rule 15b : men{V} -> me-t{V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 15b
        Rule 15b : men{V} -> me-t{V}
        """
        matches = re.match(r'^men([aiueo])(.*)$', word)
        if matches:
            return 't' + matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule3(object):
    """Disambiguate Prefix Rule 3
    Rule 3 : berCAerV -> ber-CAerV where C != 'r'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 3
        Rule 3 : berCAerV -> ber-CAerV where C != 'r'
        """
        matches = re.match(r'ber([bcdfghjklmnpqrstvwxyz])([a-z])er([aiueo])(.*)', word)
        if matches:
            if matches.group(1) == 'r':
                return
            
            return matches.group(1) + matches.group(2) + 'er' + matches.group(3) + matches.group(4)



import os
from Sastrawi.Dictionary.ArrayDictionary import ArrayDictionary
from Sastrawi.Stemmer.Stemmer import Stemmer
from Sastrawi.Stemmer.CachedStemmer import CachedStemmer
from Sastrawi.Stemmer.Cache.ArrayCache import ArrayCache

class StemmerFactory(object):
    """ Stemmer factory helps creating pre-configured stemmer """
    APC_KEY = 'sastrawi_cache_dictionary'

    def create_stemmer(self, isDev=False):
        """ Returns Stemmer instance """

        words = self.get_words(isDev)
        dictionary = ArrayDictionary(words)
        stemmer = Stemmer(dictionary)

        resultCache = ArrayCache()
        cachedStemmer = CachedStemmer(resultCache, stemmer)

        return cachedStemmer

    def get_words(self, isDev=False):
        #if isDev or callable(getattr(self, 'apc_fetch')):
        #    words = self.getWordsFromFile()
        #else:
        #    words = apc_fetch(self.APC_KEY)
        #    if not words:
        #        words = self.getWordsFromFile()
        #        apc_store(self.APC_KEY, words)
        return self.get_words_from_file()

    def get_words_from_file(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        dictionaryFile = current_dir + '/data/kata-dasar.txt'
        if not os.path.isfile(dictionaryFile):
            raise RuntimeError('Dictionary file is missing. It seems that your installation is corrupted.')

        dictionaryContent = ''
        with open(dictionaryFile, 'r') as f:
            dictionaryContent = f.read()

        return dictionaryContent.split('\n')


import re

class DisambiguatorPrefixRule39a(object):
    """Disambiguate Prefix Rule 39a (CC infix rules)
    Rule 39a : CemV -> CemV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 39a (CC infix rules)
        Rule 39a : CemV -> CemV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])(em[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)

class DisambiguatorPrefixRule39b(object):
    """Disambiguate Prefix Rule 39b (CC infix rules)
    Rule 39b : CemV -> CV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 39b (CC infix rules)
        Rule 39b : CemV -> CV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])em([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule6a(object):
    """Disambiguate Prefix Rule 6a
    Rule 6a : terV -> ter-V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 6a
        Rule 6a : terV -> ter-V
        """
        matches = re.match(r'^ter([aiueo].*)$', word)
        if matches:
            return matches.group(1)

class DisambiguatorPrefixRule6b(object):
    """Disambiguate Prefix Rule 6b
    Rule 6b : terV -> te-rV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 6b
        Rule 6b : terV -> te-rV
        """
        matches = re.match(r'^ter([aiueo].*)$', word)
        if matches:
            return 'r' + matches.group(1)



import re

class DisambiguatorPrefixRule20(object):
    """Disambiguate Prefix Rule 20
    Rule 20 : pe{w|y}V -> pe-{w|y}V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 20
        Rule 20 : pe{w|y}V -> pe-{w|y}V
        """
        matches = re.match(r'^pe([wy])([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule38a(object):
    """Disambiguate Prefix Rule 38a (CC infix rules)
    Rule 38a : CelV -> CelV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 38a (CC infix rules)
        Rule 38a : CelV -> CelV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])(el[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)

class DisambiguatorPrefixRule38b(object):
    """Disambiguate Prefix Rule 38b (CC infix rules)
    Rule 38b : CelV -> CV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 38b (CC infix rules)
        Rule 38b : CelV -> CV
        """
        matches = re.match(r'^([bcdfghjklmnpqrstvwxyz])el([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class InvalidAffixPairSpecification(object):
    """Asian J. (2007) "Effective Techniques for Indonesian Text Retrieval". page 26
    
    @link http://researchbank.rmit.edu.au/eserv/rmit:6312/Asian.pdf
    """
    def is_satisfied_by(self, word):
        if re.match(r'^me(.*)kan$', word):
            return False

        if word == 'ketahui':
            return False

        invalid_affixes = [r'^ber(.*)i$',
            r'^di(.*)an$',
            r'^ke(.*)i$',
            r'^ke(.*)an$',
            r'^me(.*)an$',
            r'^me(.*)an$',
            r'^ter(.*)an$',
            r'^per(.*)an$']

        contains = False
        for invalidAffix in invalid_affixes:
            contains = contains or re.match(invalidAffix, word)

        return contains





import re

class DisambiguatorPrefixRule16(object):
    """Disambiguate Prefix Rule 16
    Original Nazief and Adriani's Rule 16 : meng{g|h|q} -> meng-{g|h|q}
    Modified Jelita Asian's CS Rule 16 : meng{g|h|q|k} -> meng-{g|h|q|k} to stem mengkritik
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 16
        Original Nazief and Adriani's Rule 16 : meng{g|h|q} -> meng-{g|h|q}
        Modified Jelita Asian's CS Rule 16 : meng{g|h|q|k} -> meng-{g|h|q|k} to stem mengkritik
        """
        matches = re.match(r'^meng([g|h|q|k])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



#from Sastrawi.Stemmer.StemmerInterface import StemmerInterface
from Sastrawi.Stemmer.Filter import TextNormalizer

class CachedStemmer(object):
    """description of class"""
    def __init__(self, cache, delegatedStemmer):
        self.cache = cache
        self.delegatedStemmer = delegatedStemmer

    def stem(self, text):
        normalizedText = TextNormalizer.normalize_text(text)

        words = normalizedText.split(' ')
        stems = []

        for word in words:
            if self.cache.has(word):
                stems.append(self.cache.get(word))
            else:
                stem = self.delegatedStemmer.stem(word)
                self.cache.set(word, stem)
                stems.append(stem)

        return ' '.join(stems)
    
    def get_cache(self):
        return self.cache



from Sastrawi.Stemmer.ConfixStripping.PrecedenceAdjustmentSpecification \
   import PrecedenceAdjustmentSpecification

class Context(object):
    """Stemming Context using Nazief and Adriani, CS, ECS, Improved ECS"""

    def __init__(self, original_word, dictionary, visitor_provider):
        self.original_word = original_word
        self.current_word = original_word
        self.dictionary = dictionary
        self.visitor_provider = visitor_provider

        self.process_is_stopped = False
        self.removals = []
        self.visitors = []
        self.suffix_visitors = []
        self.prefix_pisitors = []
        self.result = ''

        self.init_visitors()

    def init_visitors(self):
        self.visitors = self.visitor_provider.get_visitors()
        self.suffix_visitors = self.visitor_provider.get_suffix_visitors()
        self.prefix_pisitors = self.visitor_provider.get_prefix_visitors()

    def stopProcess(self):
        self.process_is_stopped = True

    def add_removal(self, removal):
        self.removals.append(removal)

    def execute(self):
        """Execute stemming process; the result can be retrieved with result"""

        #step 1 - 5
        self.start_stemming_process()

        #step 6
        if self.dictionary.contains(self.current_word):
            self.result = self.current_word
        else:
            self.result = self.original_word

    def start_stemming_process(self):

        #step 1
        if self.dictionary.contains(self.current_word):
            return
        self.accept_visitors(self.visitors)
        if self.dictionary.contains(self.current_word):
            return

        csPrecedenceAdjustmentSpecification = PrecedenceAdjustmentSpecification()

        #Confix Stripping
        #Try to remove prefix before suffix if the specification is met
        if csPrecedenceAdjustmentSpecification.is_satisfied_by(self.original_word):
            #step 4, 5
            self.remove_prefixes()
            if self.dictionary.contains(self.current_word):
                return

            #step 2, 3
            self.remove_suffixes()
            if self.dictionary.contains(self.current_word):
                return
            else:
                #if the trial is failed, restore the original word
                #and continue to normal rule precedence (suffix first, prefix afterwards)
                self.current_word = self.original_word
                self.removals = []

        #step 2, 3
        self.remove_suffixes()
        if self.dictionary.contains(self.current_word):
            return

        #step 4, 5
        self.remove_prefixes()
        if self.dictionary.contains(self.current_word):
            return

        #ECS loop pengembalian akhiran
        self.loop_pengembalian_akhiran()

    def remove_prefixes(self):
        for i in range(3):
            self.accept_prefix_visitors(self.prefix_pisitors)
            if self.dictionary.contains(self.current_word):
                return

    def remove_suffixes(self):
        self.accept_visitors(self.suffix_visitors)

    def accept(self, visitor):
        visitor.visit(self)

    def accept_visitors(self, visitors):
        for visitor in visitors:
            self.accept(visitor)
            if self.dictionary.contains(self.current_word):
                return self.current_word
            if self.process_is_stopped:
                return self.current_word

    def accept_prefix_visitors(self, visitors):
        removalCount = len(self.removals)
        for visitor in visitors:
            self.accept(visitor)
            if self.dictionary.contains(self.current_word):
                return self.current_word
            if self.process_is_stopped:
                return self.current_word
            if len(self.removals) > removalCount:
                return

    def loop_pengembalian_akhiran(self):
        """ECS Loop Pengembalian Akhiran"""
        self.restore_prefix()

        removals = self.removals
        reversed_removals = reversed(removals)
        current_word = self.current_word

        for removal in reversed_removals:
            if not self.is_suffix_removal(removal):
                continue
            if removal.get_removed_part() == 'kan':
                self.current_word = removal.result + 'k'

                #step 4,5
                self.remove_prefixes()
                if self.dictionary.contains(self.current_word):
                    return
                self.current_word = removal.result + 'kan'
            else:
                self.current_word = removal.get_subject()

            #step 4,5
            self.remove_prefixes()
            if self.dictionary.contains(self.current_word):
                return

            self.removals = removals
            self.current_word = current_word

    def is_suffix_removal(self, removal):
        """Check wether the removed part is a suffix"""
        return removal.get_affix_type() == 'DS' \
               or removal.get_affix_type() == 'PP' \
               or removal.get_affix_type() == 'P'

    def restore_prefix(self):
        """Restore prefix to proceed with ECS loop pengembalian akhiran"""
        for removal in self.removals:
            #return the word before precoding (the subject of first prefix removal)
            self.current_word = removal.get_subject()
            break

        for removal in self.removals:
            if removal.get_affix_type() == 'DP':
                self.removals.remove(removal)




import re

class DisambiguatorPrefixRule10(object):
    """Disambiguate Prefix Rule 10
    Rule 10 : me{l|r|w|y}V -> me-{l|r|w|y}V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 10
        Rule 10 : me{l|r|w|y}V -> me-{l|r|w|y}V
        """
        matches = re.match(r'^me([lrwy])([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule28a(object):
    """Disambiguate Prefix Rule 28a
    Rule 28a : pen{V} -> pe-n{V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 28a
        Rule 28a : pen{V} -> pe-n{V}
        """
        matches = re.match(r'^pen([aiueo])(.*)$', word)
        if matches:
            return 'n' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule28b(object):
    """Disambiguate Prefix Rule 28b
    Rule 28b : pen{V} -> pe-t{V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 28b
        Rule 28b : pen{V} -> pe-t{V}
        """
        matches = re.match(r'^pen([aiueo])(.*)$', word)
        if matches:
            return 't' + matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule18a(object):
    """Disambiguate Prefix Rule 18a
    Rule 18a : menyV -> me-nyV to stem menyala -> nyala
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 18a
        Rule 18a : menyV -> me-nyV to stem menyala -> nyala
        """
        matches = re.match(r'^meny([aiueo])(.*)$', word)
        if matches:
            return 'ny' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule18b(object):
    """Disambiguate Prefix Rule 18a
    Original Rule 18 : menyV -> meny-sV
    Modified by CC (shifted into 18b, see also 18a)
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 18a
        Original Rule 18 : menyV -> meny-sV
        Modified by CC (shifted into 18b, see also 18a)
        """
        matches = re.match(r'^meny([aiueo])(.*)$', word)
        if matches:
            return 's' + matches.group(1) + matches.group(2)



class ArrayDictionary(object):
    """description of class"""

    def __init__(self, words=None):
        self.words = []
        if words:
            self.add_words(words)

    def contains(self, word):
        return word in self.words

    def count(self):
        return len(self.words)

    def add_words(self, words):
        """Add multiple words to the dictionary"""
        for word in words:
            self.add(word)

    def add(self, word):
        """Add a word to the dictionary"""
        if not word or word.strip() == '':
            return
        self.words.append(word)








class StopWordRemover(object):
    """description of class"""

    def __init__(self, dictionary):
        self.dictionary = dictionary

    def get_dictionary(self):
        return self.dictionary

    def remove(self, text):
        """Remove stop words."""
        words = text.split(' ')
        stopped_words = [word for word in words if not self.dictionary.contains(word)]

        return ' '.join(stopped_words)







import re

class DisambiguatorPrefixRule27(object):
    """Disambiguate Prefix Rule 27
    Rule 27 modified by Prasasto Adi : pen{c|d|j|s|t|z} -> pen-{c|d|j|s|t|z}
    in order to stem penstabilan, pentranskripsi

    Original CS Rule 27 was : pen{c|d|j|z} -> pen-{c|d|j|z}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 27
        Rule 27 modified by Prasasto Adi : pen{c|d|j|s|t|z} -> pen-{c|d|j|s|t|z}
        in order to stem penstabilan, pentranskripsi

        Original CS Rule 27 was : pen{c|d|j|z} -> pen-{c|d|j|z}
        """
        matches = re.match(r'^pen([cdjstz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule35(object):
    """Disambiguate Prefix Rule 35
    Rule 35 : terC1erC2 -> ter-C1erC2 where C1 != {r}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 35
        Rule 35 : terC1erC2 -> ter-C1erC2 where C1 != {r}
        """
        matches = re.match(r'^ter([bcdfghjkpqstvxz])(er[bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule25(object):
    """Disambiguate Prefix Rule 25
    Rule 25 : pem{b|f|v} -> pem-{b|f|v}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 25
        Rule 25 : pem{b|f|v} -> pem-{b|f|v}
        """
        matches = re.match(r'^pem([bfv])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule8(object):
    """Disambiguate Prefix Rule 8
    Rule 8 : terCP -> ter-CP where C != 'r' and P != 'er'
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 8
        Rule 8 : terCP -> ter-CP where C != 'r' and P != 'er'
        """
        matches = re.match(r'^ter([bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            if matches.group(1) == 'r' or re.match(r'^er(.*)$', matches.group(2)):
                return
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule36(object):
    """Disambiguate Prefix Rule 36
    Rule 36 : peC1erC2 -> pe-C1erC2 where C1 != {r|w|y|l|m|n}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 36
        Rule 36 : peC1erC2 -> pe-C1erC2 where C1 != {r|w|y|l|m|n}
        """
        matches = re.match(r'^pe([bcdfghjkpqstvxz])(er[bcdfghjklmnpqrstvwxyz])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2) + matches.group(3)



import re

class DisambiguatorPrefixRule21a(object):
    """Disambiguate Prefix Rule 21a
    Rule 21a : perV -> per-V
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 21a
        Rule 21a : perV -> per-V
        """
        matches = re.match(r'^per([aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule21b(object):
    """Disambiguate Prefix Rule 21b
    Rule 21b : perV -> pe-rV
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 21b
        Rule 21b : perV -> pe-rV
        """
        matches = re.match(r'^pe(r[aiueo])(.*)$', word)
        if matches:
            return matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule26a(object):
    """Disambiguate Prefix Rule 26a
    Rule 26a : pem{rV|V} -> pe-m{rV|V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 26a
        Rule 26a : pem{rV|V} -> pe-m{rV|V}
        """
        matches = re.match(r'^pem([aiueo])(.*)$', word)
        if matches:
            return 'm' + matches.group(1) + matches.group(2)

class DisambiguatorPrefixRule26b(object):
    """Disambiguate Prefix Rule 26b
    Rule 26b : pem{rV|V} -> pe-p{rV|V}
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 26b
        Rule 26b : pem{rV|V} -> pe-p{rV|V}
        """
        matches = re.match(r'^pem([aiueo])(.*)$', word)
        if matches:
            return 'p' + matches.group(1) + matches.group(2)



import re

class DisambiguatorPrefixRule19(object):
    """Disambiguate Prefix Rule 19
    Original Rule 19 : mempV -> mem-pV where V != 'e'
    Modified Rule 19 by ECS : mempA -> mem-pA where A != 'e' in order to stem memproteksi
    """
    
    def disambiguate(self, word):
        """Disambiguate Prefix Rule 19
        Original Rule 19 : mempV -> mem-pV where V != 'e'
        Modified Rule 19 by ECS : mempA -> mem-pA where A != 'e' in order to stem memproteksi
        """
        matches = re.match(r'^memp([abcdfghijklmopqrstuvwxyz])(.*)$', word)
        if matches:
            return 'p' + matches.group(1) + matches.group(2)
